@@ -75,7 +75,7 @@ void aio_waitable::remove_waiter(aio_waiter & h)
 		yield();
 		l.lock();
 	}
-	std::remove(waiting_.begin(), waiting_.end(), &h);
+	waiting_.erase(std::remove(waiting_.begin(), waiting_.end(), &h), waiting_.end());
 }
 
 namespace {
@@ -98,7 +98,7 @@ void aio_waitable::remove_waiter(event_handler & h)
 {
 	scoped_lock l(m_);
 	remove_pending_events(h, *this);
-	std::remove(waiting_handlers_.begin(), waiting_handlers_.end(), &h);
+	waiting_handlers_.erase(std::remove(waiting_handlers_.begin(), waiting_handlers_.end(), &h), waiting_handlers_.end());
 }
 
 void aio_waitable::remove_waiters()
