@@ -55,7 +55,7 @@ result file::open(native_string const& f, mode m, creation_flags d)
 	}
 
 	DWORD dispositionFlags;
-	if (m == writing) {
+	if (m == writing || m == readwrite) {
 		if (d & empty) {
 			dispositionFlags = CREATE_ALWAYS;
 		}
@@ -88,7 +88,14 @@ result file::open(native_string const& f, mode m, creation_flags d)
 		}
 		attr.lpSecurityDescriptor = sd;
 	}
-	fd_ = CreateFile(f.c_str(), (m == reading) ? GENERIC_READ : GENERIC_WRITE, shareMode, &attr, dispositionFlags, FILE_FLAG_SEQUENTIAL_SCAN, nullptr);
+	DWORD access{};
+	if (m != writing) {
+		access |= GENERIC_READ;
+	}
+	if (m != reading) {
+		access |= GENERIC_WRITE;
+	}
+	fd_ = CreateFileW(f.c_str(), access, shareMode, &attr, dispositionFlags, FILE_FLAG_SEQUENTIAL_SCAN, nullptr);
 
 	if (fd_ == INVALID_HANDLE_VALUE) {
 		auto const err = GetLastError();
@@ -247,7 +254,8 @@ result file::open(native_string const& f, mode m, creation_flags d)
 		flags |= O_RDONLY;
 	}
 	else {
-		flags |= O_WRONLY | O_CREAT;
+		flags |= (m == writing) ? O_WRONLY : O_RDWR;
+		flags |= O_CREAT;
 		if (d & empty) {
 			flags |= O_TRUNC;
 		}
