@@ -351,8 +351,7 @@ native_string current_username()
 
 #elif FZ_WINDOWS
 
-#include "windows/dll.hpp"
-
+#include "libfilezilla/glue/dll.hpp"
 #include "libfilezilla/glue/windows.hpp"
 #include "windows/security_descriptor_builder.hpp"
 
@@ -460,9 +459,8 @@ bool impersonation_token::operator<(impersonation_token const& op) const
 
 namespace {
 extern "C" {
-typedef HRESULT (*getknownfolderpath_t)(REFKNOWNFOLDERID, DWORD, HANDLE, wchar_t**);
-typedef HRESULT (*getknownfolderpath_t)(REFKNOWNFOLDERID, DWORD, HANDLE, wchar_t**);
-typedef void (*cotaskmemfree_t)(void*);
+typedef HRESULT (STDAPICALLTYPE *getknownfolderpath_t)(REFKNOWNFOLDERID, DWORD, HANDLE, wchar_t**);
+typedef void (STDAPICALLTYPE *cotaskmemfree_t)(void*);
 }
 }
 
@@ -476,8 +474,8 @@ native_string impersonation_token::home() const
 		static GUID const profile = { 0x5E6C858F, 0x0E22, 0x4760, {0x9A, 0xFE, 0xEA, 0x33, 0x17, 0xB6, 0x71, 0x73} };
 
 		static auto& dlls = shdlls::get();
-		static getknownfolderpath_t const getknownfolderpath = dlls.shell32_ ? reinterpret_cast<getknownfolderpath_t>(GetProcAddress(dlls.shell32_.h_, "SHGetKnownFolderPath")) : nullptr;
-		static cotaskmemfree_t const cotaskmemfree = dlls.ole32_ ? reinterpret_cast<cotaskmemfree_t>(GetProcAddress(dlls.ole32_.h_, "CoTaskMemFree")) : nullptr;
+		static auto const getknownfolderpath = reinterpret_cast<getknownfolderpath_t>(dlls.shell32_["SHGetKnownFolderPath"]);
+		static auto const cotaskmemfree = reinterpret_cast<cotaskmemfree_t>(dlls.ole32_["CoTaskMemFree"]);
 
 		if (getknownfolderpath && cotaskmemfree && getknownfolderpath(profile, 0, impl_->h_, &out) == S_OK) {
 			ret = out;
