@@ -387,24 +387,7 @@ std::pair<std::string, bool> json_unescape_string(char const*& p, char const* en
 						return {};
 					}
 
-					if (u <= 0x7f) {
-						ret += static_cast<char>(u);
-					}
-					else if (u <= 0x7ff) {
-						ret += static_cast<char>(0xc0u | ((u >> 6u) & 0x1fu));
-						ret += static_cast<char>(0x80u | (u & 0x3fu));
-					}
-					else if (u <= 0xffff) {
-						ret += static_cast<char>(0xe0u | ((u >> 12u) & 0x0fu));
-						ret += static_cast<char>(0x80u | ((u >> 6u) & 0x3fu));
-						ret += static_cast<char>(0x80u | (u & 0x3fu));
-					}
-					else {
-						ret += static_cast<char>(0xf0u | ((u >> 18u) & 0x07u));
-						ret += static_cast<char>(0x80u | ((u >> 12u) & 0x3fu));
-						ret += static_cast<char>(0x80u | ((u >> 6u) & 0x3fu));
-						ret += static_cast<char>(0x80u | (u & 0x3fu));
-					}
+					unicode_codepoint_to_utf8_append(ret, u);
 					break;
 				}
 				default:
