@@ -21,9 +21,16 @@
 struct sockaddr;
 
 namespace fz {
+class buffer;
 class thread_pool;
 
-/// The type of a socket event
+/** \brief The type of a socket event
+ *
+ * In received events, exactly a single bit is always set.
+ *
+ * Flag combinations are used when changing event handlers,
+ * \sa f::socket::set_event_handler
+ */
 enum class socket_event_flag
 {
 	/**
@@ -31,27 +38,41 @@ enum class socket_event_flag
 	 * additional addresses to try.
 	 * Errors can be ignored for this type.
 	 */
-	connection_next,
+	connection_next = 0x1,
 
 	/**
 	 * If no error is set, the connection has been established.
 	 * If an error is set, the connection could not be established.
 	 */
-	connection,
+	connection = 0x2,
 
 	/**
 	 * If no error is set, data has become available.
 	 * If an error is set, the connection has failed.
 	 */
-	read,
+	read = 0x4,
 
 	/**
 	 * If no error is set, data can be written.
 	 * If an error is set, the connection has failed.
 	 */
-	write
+	write = 0x8,
 };
 
+inline bool operator&(socket_event_flag lhs, socket_event_flag rhs) {
+	return (static_cast<std::underlying_type_t<socket_event_flag>>(lhs) & static_cast<std::underlying_type_t<socket_event_flag>>(rhs)) != 0;
+}
+inline socket_event_flag operator|(socket_event_flag lhs, socket_event_flag rhs)
+{
+	return static_cast<socket_event_flag>(static_cast<std::underlying_type_t<socket_event_flag>>(lhs) | static_cast<std::underlying_type_t<socket_event_flag>>(rhs));
+}
+inline socket_event_flag& operator|=(socket_event_flag& lhs, socket_event_flag rhs)
+{
+	lhs = lhs | rhs;
+	return lhs;
+}
+
+
 /**
  * \brief All classes sending socket events should derive from this.
  *
@@ -74,7 +95,7 @@ public:
 	}
 
 protected:
-	socket_event_source() = default;
+	socket_event_source() = delete;
 	explicit socket_event_source(socket_event_source* root)
 		: root_(root)
 	{}
@@ -101,6 +122,8 @@ struct socket_event_type;
  *   socket_interface::shutdown_read has returned EAGAIN.
  * - The same holds for the write event and socket_interface::write and
  *   socket_interface::shutdown
+ * - A successful connection events doubles as write event, it does not
+ *   act as read event
  *
  * It is a grave violation to call the read/write/shutdown functions
  * again after they returned EAGAIN without first waiting for the event.
@@ -131,8 +154,10 @@ void FZ_PUBLIC_SYMBOL remove_socket_events(event_handler * handler, socket_event
  * This function is called by socket::set_event_handler().
  *
  * Example use-cases: Handoff after proxy handshakes, or handoff to TLS classes in case of STARTTLS mechanism
+ *
+ * Returns which events are still pending.
  */
-void FZ_PUBLIC_SYMBOL change_socket_event_handler(event_handler * old_handler, event_handler * new_handler, socket_event_source const* const source);
+fz::socket_event_flag FZ_PUBLIC_SYMBOL change_socket_event_handler(event_handler * old_handler, event_handler * new_handler, socket_event_source const* const source, fz::socket_event_flag remove);
 
 /// \private
 class socket_thread;
@@ -150,7 +175,7 @@ public:
 	 */
 	int set_buffer_sizes(int size_receive, int size_send);
 
-	/// If connected, either ipv4 or ipv6, unknown otherwise
+	/// If connected, either ipv4, ipv6 or unix, unknown otherwise
 	address_type address_family() const;
 
 	/**
@@ -191,8 +216,6 @@ protected:
 
 	int close();
 
-	void do_set_event_handler(event_handler* pEvtHandler);
-
 	// Note: Unlocks the lock.
 	void detach_thread(scoped_lock & l);
 
@@ -248,6 +271,20 @@ public:
 
 	explicit operator bool() const { return fd_ != -1; }
 
+	/**
+	 * \brief Returns remote address of a connected socket
+	 *
+	 * \return empty string on error
+	 */
+	std::string peer_ip(bool strip_zone_index = false) const;
+
+	/**
+	 * \brief Returns remote port of a connected socket
+	 *
+	 * \return -1 on error
+	 */
+	int peer_port(int& error) const;
+
 private:
 	socket_base::socket_t fd_{-1};
 };
@@ -277,11 +314,10 @@ public:
 	 *
 	 * The address type, if not fz::address_type::unknown, must may the type of the address passed to bind()
 	 */
-
 	int listen(address_type family, int port = 0);
 
 	/// Accepts incoming connection. If no socket is returned, error contains the reason
-	std::unique_ptr<socket> accept(int& error);
+	std::unique_ptr<socket> accept(int& error, fz::event_handler * handler = nullptr);
 
 	/**
 	 * \brief  Like accept, but only returns a socket descriptor.
@@ -293,9 +329,7 @@ public:
 
 	listen_socket_state get_state() const;
 
-	void set_event_handler(event_handler* pEvtHandler) {
-		do_set_event_handler(pEvtHandler);
-	}
+	void set_event_handler(event_handler* pEvtHandler);
 
 private:
 	listen_socket_state state_{};
@@ -344,7 +378,45 @@ public:
 	virtual int read(void* buffer, unsigned int size, int& error) = 0;
 	virtual int write(void const* buffer, unsigned int size, int& error) = 0;
 
-	virtual void set_event_handler(event_handler* pEvtHandler) = 0;
+	template<typename T, std::enable_if_t<std::is_signed_v<T>, int> = 0>
+	int read(void* buffer, T size, int& error)
+	{
+		if (size < 0) {
+			error = EINVAL;
+			return -1;
+		}
+
+		return read(buffer, static_cast<unsigned int>(size), error);
+	}
+	template<typename T, std::enable_if_t<std::is_unsigned_v<T> && (sizeof(T) > sizeof(unsigned int)), int> = 0>
+	int read(void* buffer, T size, int& error)
+	{
+		if (size > std::numeric_limits<unsigned int>::max()) {
+			size = std::numeric_limits<unsigned int>::max();
+		}
+		return read(buffer, static_cast<unsigned int>(size), error);
+	}
+
+	template<typename T, std::enable_if_t<std::is_signed_v<T>, int> = 0>
+	int write(void const* buffer, T size, int& error)
+	{
+		if (size < 0) {
+			error = EINVAL;
+			return -1;
+		}
+
+		return write(buffer, static_cast<std::make_unsigned_t<T>>(size), error);
+	}
+	template<typename T, std::enable_if_t<std::is_unsigned_v<T> && (sizeof(T) > sizeof(unsigned int)), int> = 0>
+	int write(void const* buffer, T size, int& error)
+	{
+		if (size > std::numeric_limits<unsigned int>::max()) {
+			size = std::numeric_limits<unsigned int>::max();
+		}
+		return write(buffer, static_cast<unsigned int>(size), error);
+	}
+
+	virtual void set_event_handler(event_handler* pEvtHandler, fz::socket_event_flag retrigger_block = fz::socket_event_flag{}) = 0;
 
 	virtual native_string peer_host() const = 0;
 	virtual int peer_port(int& error) const = 0;
@@ -369,7 +441,7 @@ public:
 	virtual int shutdown_read() = 0;
 
 protected:
-	socket_interface() = default;
+	socket_interface() = delete;
 
 	explicit socket_interface(socket_event_source * root)
 		: socket_event_source(root)
@@ -394,7 +466,7 @@ public:
 	socket(socket const&) = delete;
 	socket& operator=(socket const&) = delete;
 
-	static std::unique_ptr<socket> from_descriptor(socket_descriptor && desc, thread_pool & pool, int & error);
+	static std::unique_ptr<socket> from_descriptor(socket_descriptor && desc, thread_pool & pool, int & error, fz::event_handler * handler = nullptr);
 
 	socket_state get_state() const override;
 	bool is_connected() const {
@@ -452,20 +524,20 @@ public:
 	virtual int write(void const* buffer, unsigned int size, int& error) override;
 
 	/**
-	* \brief Returns remote address of a connected socket
-	*
-	* \return empty string on error
-	*/
+	 * \brief Returns remote address of a connected socket
+	 *
+	 * \return empty string on error
+	 */
 	std::string peer_ip(bool strip_zone_index = false) const;
 
 	/// Returns the hostname passed to connect()
 	virtual native_string peer_host() const override;
 
 	/**
-	* \brief Returns remote port of a connected socket
-	*
-	* \return -1 on error
-	*/
+	 * \brief Returns remote port of a connected socket
+	 *
+	 * \return -1 on error
+	 */
 	virtual int peer_port(int& error) const override;
 
 	/**
@@ -476,15 +548,21 @@ public:
 	 */
 	int ideal_send_buffer_size();
 
-	/**
-	 * Allows re-triggering the read and write events.
-	 * Slow and cumbersome, use sparingly.
-	 */
-	void retrigger(socket_event_flag event);
-
 	virtual int shutdown() override;
 
-	virtual void set_event_handler(event_handler* pEvtHandler) override;
+	/**
+	 * \brief Changes the associated event handler.
+	 *
+	 * Pending events are rewritten to the new handler, or deleted if there is no new handler.
+	 *
+	 * Initially, the new handler is assumed to be waiting on read and write events and if the
+	 * socket is in a readable/writable state, the corresponding events are sent if not already
+	 * pending.
+	 *
+	 * As exception, events passed in retrigger_block are always removed and not resent if the socket
+	 * is in the readable/writable state.
+	 */
+	virtual void set_event_handler(event_handler* pEvtHandler, fz::socket_event_flag retrigger_block = fz::socket_event_flag{}) override;
 
 	enum
 	{
@@ -512,6 +590,38 @@ public:
 
 	virtual int shutdown_read() override { return 0; }
 
+	socket_t get_descriptor();
+
+#ifndef FZ_WINDOWS
+	/** Sends file descriptors over a Unix Domain Socket.
+	 *
+	 * fd may be -1.
+	 * If fd is not -1, the buffer must not be empty.
+	 *
+	 * Returns the amount of bytes sent, which may be less than requested, or -1 on error.
+	 *
+	 * If any bytes got sent then the descriptor has been sent as well.
+	 *
+	 * If having sent an fd, you should not send any other fd until the buffer has been completely
+	 * sent.
+	 *
+	 * The data you sent should be structured such that the receiving end can detect which
+	 * parts of data in the stream have a descriptor associated with it.
+	 */
+	int send_fd(fz::buffer & buf, int fd, int & error);
+
+	/** Reads data and file descriptors from a Unix Domain Socket
+	 *
+	 * Appends any read data to the buffer, returns the number of bytes added.
+	 *
+	 * Returns 0 on EOF, -1 on error.
+	 *
+	 * If a descriptor got read, it is returned in the fd argument. If no descriptor got read,
+	 * fd is set to -1.
+	 */
+	int read_fd(fz::buffer & buf, int &fd, int & error);
+#endif
+
 private:
 	friend class socket_base;
 	friend class listen_socket;
@@ -548,7 +658,7 @@ public:
 	socket_layer& operator=(socket_layer const&) = delete;
 
 	/// The handler for any events generated (or forwarded) by this layer.
-	virtual void set_event_handler(event_handler* handler) override;
+	virtual void set_event_handler(event_handler* handler, fz::socket_event_flag retrigger_block = fz::socket_event_flag{}) override;
 
 	/**
 	 * Can be overridden to return something different, e.g. a proxy layer
@@ -590,6 +700,18 @@ public:
 	 */
 	virtual int shutdown_read() override;
 
+	virtual int connect(native_string const& host, unsigned int port, address_type family = address_type::unknown) override {
+		return next_layer_.connect(host, port, family);
+	}
+
+	virtual int shutdown() override {
+		return next_layer_.shutdown();
+	}
+
+	virtual socket_state get_state() const override {
+		return next_layer_.get_state();
+	}
+
 protected:
 	/**
 	 * Call in a derived classes handler for fz::socket_event. Results in
@@ -609,7 +731,7 @@ protected:
 	 * A pass-through layer does not handle events itself. Instead any events sent by the next layer get
 	 * sent to the event handler passed to the layer.
 	 */
-	void set_event_passthrough();
+	void set_event_passthrough(socket_event_flag retrigger_block = socket_event_flag{});
 
 	event_handler* event_handler_{};
 	socket_interface& next_layer_;
@@ -634,6 +756,17 @@ native_string FZ_PUBLIC_SYMBOL socket_error_description(int error);
 
 #ifdef FZ_WINDOWS
 
+/// \private
+class FZ_PRIVATE_SYMBOL winsock_initializer final
+{
+public:
+	winsock_initializer();
+	~winsock_initializer();
+
+private:
+	bool initialized_{};
+};
+
 #ifndef EISCONN
 #define EISCONN WSAEISCONN
 #endif
