@@ -1207,11 +1207,60 @@ bool socket_base::bind(std::string const& address)
 }
 
 
+namespace {
+std::string get_peer_ip(int fd, bool strip_zone_index)
+{
+	sockaddr_storage addr;
+	socklen_t addr_len = sizeof(addr);
+	int res = getpeername(fd, (sockaddr*)&addr, &addr_len);
+	if (res) {
+		return std::string();
+	}
+
+	return socket::address_to_string((sockaddr *)&addr, addr_len, false, strip_zone_index);
+}
+
+int get_peer_port(int fd, int& error)
+{
+	sockaddr_u addr;
+	socklen_t addr_len = sizeof(addr);
+	error = getpeername(fd, &addr.sockaddr_, &addr_len);
+	if (error) {
+#ifdef FZ_WINDOWS
+		error = convert_msw_error_code(WSAGetLastError());
+#else
+		error = errno;
+#endif
+		return -1;
+	}
+
+	if (addr.storage.ss_family == AF_INET) {
+		return ntohs(addr.in4.sin_port);
+	}
+	else if (addr.storage.ss_family == AF_INET6) {
+		return ntohs(addr.in6.sin6_port);
+	}
+
+	error = EINVAL;
+	return -1;
+}
+}
+
 socket_descriptor::~socket_descriptor()
 {
 	close_socket_fd(fd_);
 }
 
+std::string socket_descriptor::peer_ip(bool strip_zone_index) const
+{
+	return get_peer_ip(fd_, strip_zone_index);
+}
+
+int socket_descriptor::peer_port(int& error) const
+{
+	return get_peer_port(fd_, error);
+}
+
 
 listen_socket::listen_socket(thread_pool & pool, event_handler* evt_handler)
 	: socket_base(pool, evt_handler, this)
@@ -1594,39 +1643,12 @@ int socket::write(void const* buffer, unsigned int size, int& error)
 
 std::string socket::peer_ip(bool strip_zone_index) const
 {
-	sockaddr_storage addr;
-	socklen_t addr_len = sizeof(addr);
-	int res = getpeername(fd_, (sockaddr*)&addr, &addr_len);
-	if (res) {
-		return std::string();
-	}
-
-	return address_to_string((sockaddr *)&addr, addr_len, false, strip_zone_index);
+	return get_peer_ip(fd_, strip_zone_index);
 }
 
 int socket::peer_port(int& error) const
 {
-	sockaddr_u addr;
-	socklen_t addr_len = sizeof(addr);
-	error = getpeername(fd_, &addr.sockaddr_, &addr_len);
-	if (error) {
-#ifdef FZ_WINDOWS
-		error = convert_msw_error_code(WSAGetLastError());
-#else
-		error = errno;
-#endif
-		return -1;
-	}
-
-	if (addr.storage.ss_family == AF_INET) {
-		return ntohs(addr.in4.sin_port);
-	}
-	else if (addr.storage.ss_family == AF_INET6) {
-		return ntohs(addr.in6.sin6_port);
-	}
-
-	error = EINVAL;
-	return -1;
+	return get_peer_port(fd_, error);
 }
 
 int socket::ideal_send_buffer_size()
