@@ -272,7 +272,7 @@ bool tls_layer_impl::init()
 		initialized_ = true;
 		int res = gnutls_global_init();
 		if (res) {
-			log_error(res, L"gnutls_global_init");
+			log_error(res, L"gnutls_global_init"sv);
 			deinit();
 			return false;
 		}
@@ -289,7 +289,7 @@ bool tls_layer_impl::init()
 	if (!cert_credentials_) {
 		int res = gnutls_certificate_allocate_credentials(&cert_credentials_);
 		if (res < 0) {
-			log_error(res, L"gnutls_certificate_allocate_credentials");
+			log_error(res, L"gnutls_certificate_allocate_credentials"sv);
 			deinit();
 			return false;
 		}
@@ -378,7 +378,7 @@ bool tls_layer_impl::set_certificate(std::string_view const& key, std::string_vi
 	int res = gnutls_certificate_set_x509_key_mem2(cert_credentials_, &c,
 		&k, pem ? GNUTLS_X509_FMT_PEM : GNUTLS_X509_FMT_DER, password.empty() ? nullptr : to_utf8(password).c_str(), 0);
 	if (res < 0) {
-		log_error(res, L"gnutls_certificate_set_x509_key_mem2");
+		log_error(res, L"gnutls_certificate_set_x509_key_mem2"sv);
 		deinit();
 		return false;
 	}
@@ -397,7 +397,7 @@ bool tls_layer_impl::init_session(bool client, int extra_flags)
 	flags |= extra_flags;
 	int res = gnutls_init(&session_, flags);
 	if (res) {
-		log_error(res, L"gnutls_init");
+		log_error(res, L"gnutls_init"sv);
 		deinit();
 		return false;
 	}
@@ -407,7 +407,7 @@ bool tls_layer_impl::init_session(bool client, int extra_flags)
 			datum_holder h;
 			res = gnutls_session_ticket_key_generate(&h);
 			if (res) {
-				log_error(res, L"gnutls_session_ticket_key_generate");
+				log_error(res, L"gnutls_session_ticket_key_generate"sv);
 				deinit();
 				return false;
 			}
@@ -419,7 +419,7 @@ bool tls_layer_impl::init_session(bool client, int extra_flags)
 		k.size = ticket_key_.size();
 		res = gnutls_session_ticket_enable_server(session_, &k);
 		if (res) {
-			log_error(res, L"gnutls_session_ticket_enable_server");
+			log_error(res, L"gnutls_session_ticket_enable_server"sv);
 			deinit();
 			return false;
 		}
@@ -474,7 +474,7 @@ bool tls_layer_impl::init_session(bool client, int extra_flags)
 
 	res = gnutls_priority_set_direct(session_, prio.c_str(), nullptr);
 	if (res) {
-		log_error(res, L"gnutls_priority_set_direct");
+		log_error(res, L"gnutls_priority_set_direct"sv);
 		deinit();
 		return false;
 	}
@@ -534,7 +534,28 @@ void tls_layer_impl::deinit_session()
 }
 
 
-void tls_layer_impl::log_error(int code, std::wstring const& function, logmsg::type logLevel)
+void tls_layer_impl::log_gnutls_error(logger_interface &logger, int code, std::wstring_view const& function, logmsg::type logLevel)
+{
+	char const* error = gnutls_strerror(code);
+	if (error) {
+		if (function.empty()) {
+			logger.log(logLevel, fztranslate("GnuTLS error %d: %s"), code, error);
+		}
+		else {
+			logger.log(logLevel, fztranslate("GnuTLS error %d in %s: %s"), code, function, error);
+		}
+	}
+	else {
+		if (function.empty()) {
+			logger.log(logLevel, fztranslate("GnuTLS error %d"), code);
+		}
+		else {
+			logger.log(logLevel, fztranslate("GnuTLS error %d in %s"), code, function);
+		}
+	}
+}
+
+void tls_layer_impl::log_error(int code, std::wstring_view const& function, logmsg::type logLevel)
 {
 	if (logLevel < logmsg::debug_warning && state_ >= socket_state::shut_down && shutdown_silence_read_errors_) {
 		logLevel = logmsg::debug_warning;
@@ -560,23 +581,7 @@ void tls_layer_impl::log_error(int code, std::wstring const& function, logmsg::t
 		}
 	}
 	else {
-		char const* error = gnutls_strerror(code);
-		if (error) {
-			if (function.empty()) {
-				logger_.log(logLevel, fztranslate("GnuTLS error %d: %s"), code, error);
-			}
-			else {
-				logger_.log(logLevel, fztranslate("GnuTLS error %d in %s: %s"), code, function, error);
-			}
-		}
-		else {
-			if (function.empty()) {
-				logger_.log(logLevel, fztranslate("GnuTLS error %d"), code);
-			}
-			else {
-				logger_.log(logLevel, fztranslate("GnuTLS error %d in %s"), code, function);
-			}
-		}
+		log_gnutls_error(logger_, code, function, logLevel);
 	}
 }
 
@@ -1107,7 +1112,7 @@ int tls_layer_impl::read(void *buffer, unsigned int len, int& error)
 		error = EAGAIN;
 	}
 	else {
-		failure(res, false, L"gnutls_record_recv");
+		failure(res, false, L"gnutls_record_recv"sv);
 		error = socket_error_ ? socket_error_ : ECONNABORTED;
 	}
 
@@ -1116,7 +1121,6 @@ int tls_layer_impl::read(void *buffer, unsigned int len, int& error)
 
 int tls_layer_impl::write(void const* buffer, unsigned int len, int& error)
 {
-//	for(size_t i = 0; i < 20; ++i) {logger_.log(logmsg::error, "Why not Zoidberg?");}
 	if (state_ == socket_state::connecting) {
 		error = EAGAIN;
 		return -1;
@@ -1173,12 +1177,12 @@ int tls_layer_impl::write(void const* buffer, unsigned int len, int& error)
 		res = GNUTLS_E_PUSH_ERROR;
 	}
 
-	failure(static_cast<int>(res), false, L"gnutls_record_send");
+	failure(static_cast<int>(res), false, L"gnutls_record_send"sv);
 	error = socket_error_ ? socket_error_ : ECONNABORTED;
 	return -1;
 }
 
-void tls_layer_impl::failure(int code, bool send_close, std::wstring const& function)
+void tls_layer_impl::failure(int code, bool send_close, std::wstring_view const& function)
 {
 	logger_.log(logmsg::debug_debug, L"tls_layer_impl::failure(%d)", code);
 	if (code) {
@@ -1263,7 +1267,7 @@ int tls_layer_impl::continue_shutdown()
 			res = GNUTLS_E_PUSH_ERROR;
 		}
 		if (res) {
-			failure(res, false, L"gnutls_bye");
+			failure(res, false, L"gnutls_bye"sv);
 			return socket_error_ ? socket_error_ : ECONNABORTED;
 		}
 		sent_closure_alert_ = true;
@@ -1717,7 +1721,7 @@ int tls_layer_impl::verify_certificate()
 		datum_holder cert_der{};
 		int res = gnutls_x509_crt_export2(certs.certs[0], GNUTLS_X509_FMT_DER, &cert_der);
 		if (res != GNUTLS_E_SUCCESS) {
-			failure(res, true, L"gnutls_x509_crt_export2");
+			failure(res, true, L"gnutls_x509_crt_export2"sv);
 			return ECONNABORTED;
 		}
 
@@ -2129,7 +2133,7 @@ void tls_layer_impl::set_hostname(native_string const& host)
 		if (!utf8.empty()) {
 			int res = gnutls_server_name_set(session_, GNUTLS_NAME_DNS, utf8.c_str(), utf8.size());
 			if (res) {
-				log_error(res, L"gnutls_server_name_set", logmsg::debug_warning);
+				log_error(res, L"gnutls_server_name_set"sv, logmsg::debug_warning);
 			}
 		}
 	}
@@ -2537,7 +2541,7 @@ bool tls_layer_impl::do_set_alpn()
 	delete [] data;
 
 	if (res) {
-		log_error(res, L"gnutls_alpn_set_protocols");
+		log_error(res, L"gnutls_alpn_set_protocols"sv);
 	}
 	return res == 0;
 }
@@ -2603,7 +2607,7 @@ int tls_layer_impl::new_session_ticket()
 		return 0;
 	}
 
-	failure(res, false, L"gnutls_session_ticket_send");
+	failure(res, false, L"gnutls_session_ticket_send"sv);
 	return socket_error_ ? socket_error_ : ECONNABORTED;
 }
 
