@@ -15,6 +15,7 @@ typedef std::make_signed_t<size_t> ssize_t;
 #include "libfilezilla/tls_layer.hpp"
 
 #include <optional>
+#include <string_view>
 
 namespace fz {
 class tls_system_trust_store;
@@ -95,6 +96,7 @@ public:
 
 	static int load_certificates(std::string_view const& in, bool pem, gnutls_x509_crt_t *& certs, unsigned int & certs_size, bool & sort);
 	static bool extract_cert(gnutls_x509_crt_t const& cert, x509_certificate& out, bool last, logger_interface * logger);
+	static void log_gnutls_error(logger_interface &logger, int code, std::wstring_view const& function = {}, logmsg::type logLevel = logmsg::error);
 
 	void set_min_tls_ver(tls_ver ver);
 	void set_max_tls_ver(tls_ver ver);
@@ -116,11 +118,11 @@ private:
 	bool certificate_is_blacklisted(cert_list_holder const& certificates);
 	bool certificate_is_blacklisted(gnutls_x509_crt_t const& cert);
 
-	void log_error(int code, std::wstring const& function, logmsg::type logLevel = logmsg::error);
+	void log_error(int code, std::wstring_view const& function, logmsg::type logLevel = logmsg::error);
 	void log_alert(logmsg::type logLevel);
 
 	// Failure logs the error, uninits the session and sends a close event
-	void failure(int code, bool send_close, std::wstring const& function = std::wstring());
+	void failure(int code, bool send_close, std::wstring_view const& function = {});
 
 	int do_call_gnutls_record_recv(void* data, size_t len);
 
